﻿using UnityEngine;
using System.Collections;

public class PlayerController : MonoBehaviour {

	public float 		Accel		= 0.2f;
	public float 		MaxSpeed	= 10.0f;
	public float 		Handling 	= 30.0f;
	public float 		JumpForce 	= 10.0f;
	public float		DoubleTapInterval	 = 0.2f;

	private float 		m_brake = 0.1f;
	private bool		m_isJump = false;
	private Rigidbody 	m_rigidBody;
	private Animator	m_animator;
	private float		m_upArrowKeyDownTime = 0;
	private float		m_upArrowKeyUpTime = 0;
	private bool 		m_running = false;


	float m_Accell = 0;

	// Use this for initialization
	void Start () {
		m_rigidBody = transform.GetComponent<Rigidbody> ();
		m_animator 	= gameObject.GetComponent<Animator> ();

		m_brake = MaxSpeed / 10.0f;
	}
	
	// Update is called once per frame
	void Update () {

		// 前後移動
		Move ();

		// 回転
		Rotation ();

		// ジャンプ
		Jump ();


		if (Input.GetKey (KeyCode.E))
		{
			m_animator.SetBool ("EXTREAM", true);
		}
		else
		{
			m_animator.SetBool ("EXTREAM", false);
		}

	}


	/// <summary>
	/// 前後移動.
	/// </summary>
	private void Move ()
	{
		// 前後移動
		if (Input.GetKey (KeyCode.UpArrow))
		{
			m_Accell += Accel;
			if (m_Accell > 1.0f)
			{
				m_Accell = 1.0f;
			}


			// ダブルタップ判定
			float lastUpArrowKeyDownTime = m_upArrowKeyDownTime;
			m_upArrowKeyDownTime = Time.time;
			if (m_upArrowKeyDownTime > m_upArrowKeyUpTime && m_upArrowKeyUpTime > lastUpArrowKeyDownTime) {
				if (m_upArrowKeyDownTime - lastUpArrowKeyDownTime < DoubleTapInterval) {
					m_running = true;
				}
			}
		}
		else if (Input.GetKey (KeyCode.DownArrow))
		{
			m_Accell -= Accel;
			if (m_Accell < -1.0f)
			{
				m_Accell = -1.0f;
			}


			m_running = false;
		}
		else
		{

			if (m_Accell > 0)
			{
				m_Accell -= m_brake;
			}
			if (m_Accell < 0)
			{
				m_Accell += m_brake;
			}

			if (Mathf.Abs (m_Accell) <= m_brake)
			{
				m_Accell = 0f;
			}

			m_upArrowKeyUpTime = Time.time;

		}

		float speedMatrix = 1.0f;
		if (m_running)
		{
			speedMatrix = 2.0f;
			if (m_Accell < 1.0f) {
				m_Accell = 1.0f;
			}
		}
		float speed = m_Accell * MaxSpeed * speedMatrix;
		transform.position += transform.TransformDirection (Vector3.forward) * speed * Time.deltaTime;

		// Animatorに移動速度をセット
		m_animator.SetFloat ("SPEED", Mathf.Abs(speed));

		// アニメーション速度
		if (m_Accell > 0) {
			m_animator.speed = 0.5f + (m_Accell * 0.5f);
		}
		else {
			m_animator.speed = 1.0f;
		}

	}


	/// <summary>
	/// 回転.
	/// </summary>
	private void Rotation()
	{
		// 回転
		Vector3 rot = transform.rotation.eulerAngles;
		float handle = 0f;
		if (Input.GetKey (KeyCode.RightArrow))
		{
			handle = Handling * Time.deltaTime;
			rot.y += handle;
		}
		if (Input.GetKey (KeyCode.LeftArrow))
		{
			handle = Handling * Time.deltaTime;
			rot.y -= handle;
		}
		transform.rotation = Quaternion.Euler (rot);

		// Animatorに回転をセット
		m_animator.SetFloat ("ROTATION", Mathf.Abs (handle));
	}


	private void Jump()
	{
		// ジャンプ
		if (Input.GetKey (KeyCode.Space))
		{
			if (!m_isJump)
			{
				m_rigidBody.AddForce (Vector3.up * JumpForce, ForceMode.Impulse);
			}
			m_isJump = true;
		}
		else
		{
			m_isJump = false;
		}
	}
}
